package ch.unige.solidify.config;

import javax.jms.ConnectionFactory;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.jms.DefaultJmsListenerContainerFactoryConfigurer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.jms.config.DefaultJmsListenerContainerFactory;
import org.springframework.jms.core.JmsTemplate;

@Configuration
public class JmsConfig {

  // Value in milliseconds set to one week
  private static final long JMS_MESSAGE_TIME_TO_LIVE = (long) 1000 * 60 * 60 * 24 * 7;

  @Value("${solidify.queue.transacted-session:false}")
  private boolean queueTransactedSession;

  @Value("${solidify.topic.transacted-session:false}")
  private boolean topicTransactedSession;

  @Bean
  public DefaultJmsListenerContainerFactory topicListenerFactory(
          ConnectionFactory connectionFactory,
          DefaultJmsListenerContainerFactoryConfigurer configurer) {

    DefaultJmsListenerContainerFactory factory = new DefaultJmsListenerContainerFactory();

    // This provides all boot's default to this factory, including the message converter
    configurer.configure(factory, connectionFactory);

    // listen to Topics
    factory.setPubSubDomain(true);

    return factory;
  }

  @Bean
  @Primary
  public JmsTemplate queueJmsTemplate(ConnectionFactory connectionFactory) {
    JmsTemplate jmsTemplate = new JmsTemplate();
    jmsTemplate.setConnectionFactory(connectionFactory);
    jmsTemplate.setExplicitQosEnabled(true);
    jmsTemplate.setTimeToLive(JMS_MESSAGE_TIME_TO_LIVE);
    jmsTemplate.setSessionTransacted(queueTransactedSession);

    // send messages to Queues
    jmsTemplate.setPubSubDomain(false);

    return jmsTemplate;
  }

  @Bean
  public JmsTemplate topicJmsTemplate(ConnectionFactory connectionFactory) {

    JmsTemplate jmsTemplate = new JmsTemplate();
    jmsTemplate.setConnectionFactory(connectionFactory);
    jmsTemplate.setExplicitQosEnabled(true);
    jmsTemplate.setTimeToLive(JMS_MESSAGE_TIME_TO_LIVE);
    jmsTemplate.setSessionTransacted(topicTransactedSession);

    // send messages to Topics
    jmsTemplate.setPubSubDomain(true);

    return jmsTemplate;
  }
}
