package ch.unige.solidify.controller;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.SubResourceContainer;
import ch.unige.solidify.service.CompositeResourceService;

public abstract class AssociationController<T extends Resource<T> & SubResourceContainer, V extends Resource<V>>
        extends SubResourceController<T, V> {

  @Autowired
  protected CompositeResourceService<V> childItemService;

  @PostMapping
  public HttpEntity<List<V>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    boolean noError = true;
    T parentItem = this.itemService.findOne(parentid);

    List<V> list = new ArrayList<>();
    for (String id : ids) {
      V childItem = this.childItemService.findOne(id);
      if (childItem == null) {
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
      }
      noError &= parentItem.addItem(childItem);
      list.add(childItem);
    }

    // Save entity if no error
    if (noError) {
      this.setAuthenticatedUserProperties(parentItem);
      this.itemService.save(parentItem);
      return new ResponseEntity<>(list, HttpStatus.CREATED);
    }
    return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
  }

  @SuppressWarnings("squid:S3752")
  @RequestMapping(value = SolidifyConstants.URL_ID, method = { RequestMethod.PUT,
          RequestMethod.PATCH })
  public HttpEntity<V> update(@PathVariable String parentid, @PathVariable String id,
          @RequestBody T t2) {
    return new ResponseEntity<>(HttpStatus.NOT_IMPLEMENTED);
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return deleteList(parentid, new String[] { id });
  }

  @DeleteMapping
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    boolean noError = true;
    T parentItem = this.itemService.findOne(parentid);

    for (String id : ids) {
      V item = this.itemService.findByParentIdAndChildId(parentid, getMainResourceClass(),
              getSubResourceClass(), id);
      if (item == null) {
        return HttpStatus.NOT_FOUND;
      }
      noError &= parentItem.removeItem(item);
    }

    if (noError) {
      this.itemService.save(parentItem);
      return HttpStatus.OK;
    }
    return HttpStatus.GONE;
  }

  @Override
  protected void addLinks(String parentid, V v) {
    v.removeLinks();
    v.addLinks(linkTo(methodOn(this.getClass()).list(parentid, null, null)), false, false);
  }
}
