package ch.unige.solidify.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.CompositeResource;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.SubResourceWithRelationContainer;

public abstract class Relation2TiersController<T extends Resource<T> & SubResourceWithRelationContainer, V extends Resource<V>, Z extends CompositeResource<?>>
        extends RelationController<T, V, Z> {

  @SuppressWarnings("unchecked")
  @GetMapping
  public HttpEntity<Collection<?>> list(@PathVariable String parentid, Pageable pageable) {
    T mainResourceItem = this.itemService.findOne(parentid);
    Page<V> childList = relationRepository.findByCompositeId(getCompositeResourceClass(),
            getMainResourceClass(), parentid, pageable);
    List<Object> newList = new ArrayList<>();
    for (final V childItem : childList) {
      Z childDetail = relationRepository.findByCompositeId(this.getCompositeResourceClass(),
              this.getMainResourceClass(), mainResourceItem.getResId(), this.getSubResourceClass(),
              childItem.getResId());
      Object newChild = mainResourceItem.getDetails(childItem, childDetail);
      addLinks(parentid, (V) newChild);
      newList.add(newChild);
    }

    Collection<?> collection = processPageAndLinks(newList, childList, parentid, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @PostMapping
  public HttpEntity<?> create(@PathVariable String parentid, @RequestBody String[] ids) {
    T parentItem = this.itemService.findOne(parentid);
    List<V> list = new ArrayList<>();
    for (final String i : ids) {
      V childItem = this.childItemService.findOne(i);
      if (parentItem.addItem(childItem)) {
        list.add(childItem);
      }
    }
    if (!list.isEmpty()) {
      this.itemService.save(parentItem);
      return new ResponseEntity<>(list, HttpStatus.CREATED);
    }
    return new ResponseEntity<>(HttpStatus.OK);
  }

  @SuppressWarnings("squid:S3752")
  @RequestMapping(value = SolidifyConstants.URL_ID, method = { RequestMethod.PUT,
          RequestMethod.PATCH })
  public HttpEntity<Z> update(@PathVariable String parentid, @PathVariable String id,
          @RequestBody Z newItem) {
    T parentItem = this.itemService.findOne(parentid);
    Z oldItem = relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid, this.getSubResourceClass(), id);
    if (oldItem == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (oldItem.update(newItem)) {
      this.itemService.save(parentItem);
    }
    return new ResponseEntity<>(HttpStatus.OK);
  }

  @DeleteMapping
  public HttpStatus deleteList(@PathVariable String parentid,
          @RequestBody(required = false) String[] ids) {
    T parentItem = this.itemService.findOne(parentid);
    List<V> childList = relationRepository.findByCompositeId(getCompositeResourceClass(),
            getMainResourceClass(), parentid);
    if (childList.isEmpty()) {
      return HttpStatus.GONE;
    }
    List<Z> list = new ArrayList<>();
    for (final V childItem : childList) {
      if (this.isInList(ids, childItem.getResId())) {
        Z childToDelete = relationRepository.findByCompositeId(this.getCompositeResourceClass(),
                this.getMainResourceClass(), parentid, this.getSubResourceClass(),
                childItem.getResId());
        if (childToDelete != null && parentItem.removeItem(childToDelete)) {
          list.add(childToDelete);
        }
      }
    }
    if (!list.isEmpty()) {
      this.itemService.save(parentItem);
    }
    return HttpStatus.OK;
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return this.deleteList(parentid, new String[] { id });
  }

  @GetMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<?> get(@PathVariable String parentid, @PathVariable String id) {
    T parentItem = this.itemService.findOne(parentid);
    V childItem = this.childItemService.findOne(id);
    Z childDetail = relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentItem.getResId(), this.getSubResourceClass(), id);
    if (childDetail == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    Object newChild = parentItem.getDetails(childItem, childDetail);

    addLinks(parentid, (V) newChild);
    return new ResponseEntity<>(newChild, HttpStatus.OK);
  }

}
