package ch.unige.solidify.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.CompositeResource;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.SubResourceWithRelationContainer;
import ch.unige.solidify.service.CompositeResourceService;

public abstract class Relation3TiersController<T extends Resource<T> & SubResourceWithRelationContainer, V extends Resource<V>, W extends Resource<W>, Z extends CompositeResource<?>>
        extends RelationController<T, V, Z> {

  @Autowired
  protected CompositeResourceService<W> grandChildItemService;

  @SuppressWarnings("unchecked")
  @GetMapping
  public HttpEntity<Collection<?>> list(@PathVariable String parentid, Pageable pageable) {
    T mainResourceItem = this.itemService.findOne(parentid);
    Page<V> childList = relationRepository.findByCompositeId(getCompositeResourceClass(),
            getMainResourceClass(), parentid, pageable);
    List<Object> newList = new ArrayList<>();
    for (final V childItem : childList) {
      List<W> gdChildlist = relationRepository.findByCompositeId(getCompositeResourceClass(),
              getMainResourceClass(), parentid, childItem.getResId());
      Object newChild = mainResourceItem.getDetails(childItem, gdChildlist);
      addLinks(parentid, (V) newChild);
      newList.add(newChild);
    }

    Collection<?> collection = processPageAndLinks(newList, childList, parentid, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @PostMapping
  public HttpEntity<?> create(@PathVariable String parentid, @RequestBody String[] ids) {
    T parentItem = this.itemService.findOne(parentid);
    W grandChildItem = this.grandChildItemService.findOne(parentItem.getDefault());
    List<W> list = new ArrayList<>();
    for (final String i : ids) {
      V childItem = this.childItemService.findOne(i);
      parentItem.addItem(childItem, grandChildItem);
      list.add(grandChildItem);
    }
    this.itemService.save(parentItem);
    return new ResponseEntity<>(list, HttpStatus.CREATED);
  }

  @SuppressWarnings("squid:S3752")
  @RequestMapping(value = SolidifyConstants.URL_ID, method = { RequestMethod.PUT,
          RequestMethod.PATCH })
  public HttpEntity<Z> update(@PathVariable String parentid, @PathVariable String id,
          @RequestBody Z z) {
    return new ResponseEntity<>(HttpStatus.NOT_IMPLEMENTED);
  }

  @DeleteMapping
  public HttpStatus delete(@PathVariable String parentid,
          @RequestBody(required = false) String[] ids) {
    T parentItem = this.itemService.findOne(parentid);
    List<V> childList = relationRepository.findByCompositeId(getCompositeResourceClass(),
            getMainResourceClass(), parentid);
    if (childList.isEmpty()) {
      return HttpStatus.GONE;
    }
    for (final V childItem : childList) {
      if (this.isInList(ids, childItem.getResId())) {
        List<W> gdChildlist = relationRepository.findByCompositeId(getCompositeResourceClass(),
                getMainResourceClass(), parentid, childItem.getResId());
        for (final W gdChildItem : gdChildlist) {
          parentItem.removeItem(childItem, gdChildItem);
        }
      }
    }

    this.itemService.save(parentItem);
    return HttpStatus.OK;
  }

  @GetMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<?> get(@PathVariable String parentid, @PathVariable String id) {
    T parentItem = this.itemService.findOne(parentid);
    V childItem = this.childItemService.findOne(id);
    List<W> gdChildlist = relationRepository.findByCompositeId(getCompositeResourceClass(),
            getMainResourceClass(), parentid, id);
    if (gdChildlist.isEmpty()) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    Object newChild = parentItem.getDetails(childItem, gdChildlist);
    if (newChild == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    addLinks(parentid, (V) newChild);
    return new ResponseEntity<>(newChild, HttpStatus.OK);
  }

  @PostMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<?> create(@PathVariable String parentid, @PathVariable String id,
          @RequestBody String[] ids) {
    T parentItem = this.itemService.findOne(parentid);
    V childItem = this.childItemService.findOne(id);
    List<W> list = new ArrayList<>();
    for (final String i : ids) {
      W grandChildItem = this.grandChildItemService.findOne(i);
      parentItem.addItem(childItem, grandChildItem);
      list.add(grandChildItem);
    }
    this.itemService.save(parentItem);
    return new ResponseEntity<>(list, HttpStatus.CREATED);
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id,
          @RequestBody(required = false) String[] ids) {
    T parentItem = this.itemService.findOne(parentid);
    V childItem = this.childItemService.findOne(id);
    List<W> gdChildlist = relationRepository.findByCompositeId(getCompositeResourceClass(),
            getMainResourceClass(), parentid, id);
    if (gdChildlist.isEmpty()) {
      return HttpStatus.GONE;
    }
    for (final W gdChildItem : gdChildlist) {
      if (this.isInList(ids, gdChildItem.getResId())) {
        parentItem.removeItem(childItem, gdChildItem);
      }
    }
    this.itemService.save(parentItem);
    return HttpStatus.OK;
  }
}
