package ch.unige.solidify.exception;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.PRECONDITION_FAILED;

import java.io.IOException;
import java.io.Serializable;
import java.nio.charset.Charset;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.client.DefaultResponseErrorHandler;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.unige.solidify.repository.SolidifyDbLockError;
import ch.unige.solidify.repository.SolidifyError;

public class RestResponseErrorHandler extends DefaultResponseErrorHandler {
  private static final Logger log = LoggerFactory.getLogger(RestResponseErrorHandler.class);

  @Override
  public void handleError(ClientHttpResponse response) throws IOException {
    HttpStatus statusCode = HttpStatus.resolve(response.getRawStatusCode());

    if (statusCode == PRECONDITION_FAILED || statusCode == BAD_REQUEST) {
      // we want to trait only 4XX bad request error
      byte[] bodyRawDate = getResponseBody(response);
      Charset charset = getCharset(response);
      String body = new String(bodyRawDate, charset);

      // if 412, used for Lock Exception
      if (statusCode == PRECONDITION_FAILED) {
        // first try to convert to LockException
        SolidifyDbLockError lockErrors = convertResponseBody2Exception(body,
                SolidifyDbLockError.class);
        if (lockErrors != null) {
          throw new SolidifyRestException("Lock Exception", lockErrors);
        }
      }
      // second, try to convert to SolidifyRestError
      SolidifyError error = convertResponseBody2Exception(body, SolidifyError.class);
      if (error != null) {
        throw new SolidifyRestException("Validation Exception", error);
      }
    }
    super.handleError(response);
  }

  private <T> T convertResponseBody2Exception(String body, Class<T> clazz) {
    try {
      ObjectMapper objectMapper = new ObjectMapper();
      return objectMapper.readValue(body, clazz);
    } catch (Exception e) {
      log.error("Deserialization error: target class =" + clazz + "JsonString=" + body + "", e);
      return null;
    }

  }
}
