package ch.unige.solidify.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.NoRepositoryBean;
import org.springframework.data.repository.PagingAndSortingRepository;

import ch.unige.solidify.rest.CompositeResource;

@NoRepositoryBean
public interface SolidifyCompositeRepository<V extends CompositeResource<?>>
        extends PagingAndSortingRepository<V, String> {

  // ***************
  // ** Relations **
  // ***************

  public default <V> List<V> findByCompositeId(Class<?> compositeClass, Class<?> mainClass,
          String id1, String id2) {
    throw new UnsupportedOperationException(
            "Missing implementation 'findByCompositeId' method with 2 ids ("
                    + compositeClass.getSimpleName() + " -> " + mainClass.getSimpleName() + ")");
  }

  public default <V> List<V> findByCompositeId(Class<?> compositeClass, Class<?> mainClass,
          String id) {
    throw new UnsupportedOperationException("Missing implementation 'findByCompositeId' method ("
            + compositeClass.getSimpleName() + " -> " + mainClass.getSimpleName() + ")");
  }

  public default <V> Page<V> findByCompositeId(Class<?> compositeClass, Class<?> mainClass,
          String id, Pageable pageable) {
    throw new UnsupportedOperationException(
            "Missing implementation 'findByCompositeId' pageable method ("
                    + compositeClass.getSimpleName() + " -> " + mainClass.getSimpleName() + ")");
  }

  public default <V> V findByCompositeId(Class<?> compositeClass, Class<?> mainClass1, String id1,
          Class<?> mainClass2, String id2) {
    throw new UnsupportedOperationException(
            "Missing implementation 'findByCompositeId' pageable method ("
                    + compositeClass.getSimpleName() + " -> " + mainClass1.getSimpleName() + " & "
                    + mainClass2.getSimpleName() + ")");
  }
}
