package ch.unige.solidify.repository;

import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.http.HttpStatus;
import org.springframework.web.context.request.ServletWebRequest;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;

import ch.unige.solidify.validation.FieldValidationError;
import ch.unige.solidify.validation.ValidationError;

public class SolidifyError {

  private String path;
  private HttpStatus status;
  private String error;
  private String message;
  private String timeStamp;
  private int statusCode;
  private List<FieldValidationError> validationErrors;

  @JsonIgnore
  private ValidationError validationError;

  public SolidifyError() {
  }

  public SolidifyError(ErrorAttributes errorAttributes, HttpServletRequest request) {
    Map<String, Object> mapErrorAttributes = errorAttributes
            .getErrorAttributes(new ServletWebRequest(request), false);
    if (mapErrorAttributes.containsKey("path")) {
      this.setPath((String) mapErrorAttributes.get("path"));
    } else {
      this.setPath(request.getRequestURL().toString());
    }
    this.setStatus(this.getStatus(mapErrorAttributes.get("status")));
    this.setError((String) mapErrorAttributes.get("error"));
    this.setMessage((String) mapErrorAttributes.get("message"));
    this.setTimeStamp(mapErrorAttributes.get("timestamp").toString());
    this.statusCode = this.status.value();
    if (this.validationError != null) {
      validationErrors = this.validationError.getErrors();
    }
  }

  public String getError() {
    return error;
  }

  public void setError(String error) {
    this.error = error;
  }

  public String getMessage() {
    return message;
  }

  public void setMessage(String message) {
    this.message = message;
  }

  public String getPath() {
    return path;
  }

  public void setPath(String path) {
    this.path = path;
  }

  public HttpStatus getStatus() {
    return status;
  }

  public void setStatus(HttpStatus status) {
    this.status = status;
    if (this.status != null) {
      this.statusCode = this.status.value();
    }
  }

  public String getTimeStamp() {
    return timeStamp;
  }

  public void setTimeStamp(String timeStamp) {
    this.timeStamp = timeStamp;
  }

  public ValidationError getValidationError() {
    return this.validationError;
  }

  public void setValidationError(ValidationError error) {
    this.validationError = error;
    if (this.validationError != null) {
      validationErrors = this.validationError.getErrors();
    } else {
      validationErrors = null;
    }
  }

  @JsonInclude(JsonInclude.Include.NON_EMPTY)
  public List<FieldValidationError> getValidationErrors() {
    return validationErrors;
  }

  public void setValidationErrors(List<FieldValidationError> validationErrors) {
    this.validationErrors = validationErrors;
  }

  public int getStatusCode() {
    return statusCode;
  }

  public void setStatusCode(int statusCode) {
    this.statusCode = statusCode;
  }

  private HttpStatus getStatus(Object status) {
    if (status instanceof Integer) {
      try {
        return HttpStatus.valueOf((Integer) status);
      } catch (IllegalArgumentException e) {
        // Unknown status code
        return HttpStatus.INTERNAL_SERVER_ERROR;
      }
    } else {
      return HttpStatus.INTERNAL_SERVER_ERROR;
    }
  }

}
