package ch.unige.solidify.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.NoRepositoryBean;

@NoRepositoryBean
public interface SolidifyRepositoryNoSql<T> {
  // Define Database name ~ index name
  public void setIndex(String indexName);

  // CRUD functions
  public List<T> findAll(String indexName, T search);

  public T save(T t);

  public T update(T t);

  public boolean delete(T t);

  public T findOne(String indexName, String id);

  // Default Functions
  public default Page<T> search(String indexName, String query, Pageable pageable) {
    return this.search(indexName, query, null, null, null, null, pageable);
  }

  public default Page<T> search(String indexName, String query, String set, String field,
          String from, String to, Pageable pageable) {
    return findAll(indexName, null, pageable);
  }

  public default Page<T> search(String indexName, String query, String set, String field,
          String from, String to, String filter, Pageable pageable) {
    return findAll(indexName, null, pageable);
  }

  public default Page<T> findAll(String indexName, T search, Pageable pageable) {
    List<T> list = findAll(indexName, search);
    long start = pageable.getOffset();
    long end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList((int) start, (int) end), pageable, list.size());
  }

  public default Page<T> searchRange(String indexName, String field, String from, String to,
          Pageable pageable) {
    return findAll(indexName, null, pageable);
  }
}
