package ch.unige.solidify.service;

import java.util.HashMap;
import java.util.Map;
import java.util.jar.Attributes;
import java.util.jar.Attributes.Name;
import java.util.jar.Manifest;

import javax.servlet.ServletContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.actuate.info.Info.Builder;
import org.springframework.boot.actuate.info.InfoContributor;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import ch.unige.solidify.SolidifyConstants;

@Service
public class InfoService implements InfoContributor {
  private static final Logger log = LoggerFactory.getLogger(InfoService.class);

  @Autowired
  private Environment env;
  @Autowired
  private ServletContext context;
  @Autowired
  private GitInfoService gitInfoService;

  @Override
  public void contribute(Builder builder) {
    // Application info
    Map<String, String> appDetails = new HashMap<>();
    appDetails.put("name", this.gitInfoService.getAppName());
    appDetails.put("version", this.gitInfoService.getBuildVersion());
    appDetails.put("profiles", String.join(",", env.getActiveProfiles()));
    builder.withDetail("application", appDetails);
    // Manifest
    try {
      Attributes manifest = new Manifest(context.getResourceAsStream(SolidifyConstants.MANIFEST))
              .getMainAttributes();
      Map<String, String> manifestDetails = new HashMap<>();
      addInfo(manifest, manifestDetails, Name.IMPLEMENTATION_TITLE);
      addInfo(manifest, manifestDetails, Name.IMPLEMENTATION_VERSION);
      addInfo(manifest, manifestDetails, new Name("Spring-Boot-Version"));
      if (!manifestDetails.isEmpty()) {
        builder.withDetail("manifest", manifestDetails);
      }
    } catch (Exception e) {
      log.error("Cannot read {}", SolidifyConstants.MANIFEST);
    }
  }

  private void addInfo(Attributes attrs, Map<String, String> attrsDetails, Name name) {
    try {
      attrsDetails.put(name.toString(), attrs.get(name).toString());
    } catch (NullPointerException e) {
      log.error("Cannot find {} attribute", name);
    }

  }
}
