package ch.unige.solidify.service;

import org.springframework.data.domain.Pageable;

import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.ResourceClient;

public abstract class ResourceService<T extends Resource<T>> implements ResourceClient<T> {

  protected SolidifyRestClientService restClientService;

  public ResourceService(SolidifyRestClientService restClientService) {
    this.restClientService = restClientService;
  }

  /**
   * Retrieves a RESTResource by sending a REST request to the corresponding webservice
   * 
   * @param resId
   *          The id of the RESTResource to fetch
   * @return
   */
  @Override
  public T findOne(String resId) {
    String url = this.getResourceUrl() + "/" + resId;
    return this.restClientService.getResource(url, this.getResourceClass());
  }

  /**
   * Retrieves a list of all RESTResource by sending a REST request to the corresponding webservice
   * 
   * @param pageable
   * @return
   */
  @Override
  public Collection<T> findAll(Pageable pageable) {
    return this.restClientService.getResourceList(this.getResourceUrl(), this.getResourceClass(),
            pageable);
  }

  protected abstract Class<T> getResourceClass();

  protected String getResourceUrl() {
    throw new UnsupportedOperationException(
            "Missing resource URL for type " + this.getClass().getName());
  }
}
