package ch.unige.solidify.converter;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeConverter;

import ch.unige.solidify.util.StringTool;

public class WeakManyToManyConverter implements AttributeConverter<List<String>, String> {

  public static final String SEPARATOR = ";";

  @Override
  public String convertToDatabaseColumn(List<String> ids) {

    if (ids == null || ids.isEmpty()) {
      return null;
    } else {

      /*
       * Clean values to not store any eventual empty String or duplicate value
       */
      List<String> cleanedValues = new ArrayList<>();
      for (String id : ids) {
        if (!StringTool.isNullOrEmpty(id) && !cleanedValues.contains(id)) {
          cleanedValues.add(id);
        }
      }

      return String.join(SEPARATOR, cleanedValues);
    }
  }

  @Override
  public List<String> convertToEntityAttribute(String databaseValue) {

    if (StringTool.isNullOrEmpty(databaseValue)) {
      return new ArrayList<>();
    } else {

      String[] ids = databaseValue.split(SEPARATOR);

      List<String> list = new ArrayList<>();
      for (String id : ids) {
        if (!StringTool.isNullOrEmpty(id) && !list.contains(id)) {
          list.add(id);
        }
      }

      return list;
    }

  }
}