package ch.unige.solidify.model;

import static org.springframework.util.Assert.hasText;

import java.util.Objects;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.validation.constraints.NotNull;

import org.springframework.data.jpa.domain.Specification;

import ch.unige.solidify.rest.ResourceNormalized;
import ch.unige.solidify.rest.SolidifyModuleName;
import ch.unige.solidify.specification.RoleApplicationSpecification;
import ch.unige.solidify.util.ReflectionTool;
import ch.unige.solidify.util.StringTool;

@Entity
public class RoleApplication extends ResourceNormalized<RoleApplication> {

  public static final String USER = "USER";
  public static final String ADMIN = "ADMIN";
  public static final String ROOT = "ROOT";
  public static final String TRUSTED_CLIENT = "TRUSTED_CLIENT";

  @NotNull
  @Column(unique = true)
  private String name;

  /********************************************/

  @SuppressWarnings("squid:S2637")
  public RoleApplication() {
    super();
  }

  public RoleApplication(String name) {
    super();
    hasText(name, "A role representation is required");
    setResId(name.toUpperCase());
    this.name = StringTool.toCamelCase(name);
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), name);
  }

  /********************************************/

  @Override
  public Specification<RoleApplication> getSpecification() {
    return new RoleApplicationSpecification(this);
  }

  @Override
  @SuppressWarnings("squid:S1186")
  public void init() {

  }

  @Override
  public String managedBy() {
    return SolidifyModuleName.RES_SRV;
  }

  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder();
    builder.append("Application Role [name=").append(name).append("]").append("[resId=")
            .append(getResId()).append("]");
    return builder.toString();
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (!(obj instanceof RoleApplication)) {
      return false;
    }
    RoleApplication roleApplication = (RoleApplication) obj;
    return super.equals(roleApplication) && Objects.equals(name, roleApplication.name);
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), name);
  }
}