package ch.unige.solidify.model.sso;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonFormat;

import ch.unige.solidify.model.User;
import ch.unige.solidify.util.StringTool;

/**
 * The user of the application.
 */

public class SsoUser implements User, Serializable {
  private static final long serialVersionUID = 9067304178593585691L;

  // if current application is a micro-service, the src name (for example header name) used to set
  // userContext attribute
  public static final String CONNECTED_USER_CONTEXT_SOURCE_NAME = "solidify_ms_connected_user_context";
  public static final String CONNECTED_USER_UNIQUE_ID = "solidify_ms_connected_user_id";

  protected String oauthToken;
  protected long oauthTokenExpiresTime;
  protected String firstName;
  protected String lastName;
  protected String email;
  protected String homeorganization;
  protected String homeorganizationtype;
  protected String userContext;

  @JsonFormat(with = JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)
  protected Set<String> affiliation = Collections.synchronizedSet(new HashSet<>());
  @JsonFormat(with = JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)
  protected Set<String> ismemberOf = Collections.synchronizedSet(new HashSet<>());
  @JsonFormat(with = JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)
  protected Set<String> employeeType = Collections.synchronizedSet(new HashSet<>());

  protected String unigechoucode;
  protected String unigechstudentoucode;
  protected String unigechemployeeoucode;
  protected String uniqueId;

  @JsonFormat(with = JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)
  protected Set<String> unigeChStudentCategory = Collections.synchronizedSet(new HashSet<>());
  protected String orgunitdn;
  protected String matriculationnumber;
  protected String cnindividu;
  protected String name = null;
  @JsonFormat(with = JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)
  protected Set<String> roles = Collections.synchronizedSet(new HashSet<>());
  protected long systemId = 0;

  protected Map<String, String> allProperties;

  public SsoUser() {
  }

  public SsoUser(final String name) {
    this();
    this.name = name;
  }

  public long getSystemId() {
    String uid = getUniqueId();
    if (!StringTool.isNullOrEmpty(uid) && uid.contains("@")) {
      String[] words = uid.split("@");
      String id = words[0];
      try {
        return Long.parseLong(id);

      } catch (Exception ex) {
        return 0;
      }
    }

    return 0;
  }

  public String getEmail() {
    return email;
  }

  public Set<String> getEmployeeType() {
    return employeeType;
  }

  public String getFirstName() {
    return firstName;
  }

  public String getLastName() {
    return lastName;
  }

  public String getName() {
    if (!StringTool.isNullOrEmpty(name)) {
      return name;
    }

    if (StringTool.isNullOrEmpty(firstName) && StringTool.isNullOrEmpty(lastName)) {
      return "";
    }

    if (StringTool.isNullOrEmpty(firstName)) {
      return lastName;
    }

    if (StringTool.isNullOrEmpty(lastName)) {
      return firstName;
    }

    return firstName + " " + lastName;
  }

  public String getHomeOrganization() {
    return homeorganization;
  }

  public String getHomeorganizationtype() {
    return homeorganizationtype;
  }

  public Set<String> getAffiliation() {
    return affiliation;
  }

  public Set<String> getIsmemberOf() {
    return ismemberOf;
  }

  public String getUnigechoucode() {
    return unigechoucode;
  }

  public String getUnigechstudentoucode() {
    return unigechstudentoucode;
  }

  public String getUnigechemployeeoucode() {
    return unigechemployeeoucode;
  }

  public String getUniqueId() {
    return uniqueId;
  }

  public String getAccessToken() {
    return this.oauthToken;
  }

  public long getAccessTokenExpiresTime() {
    return this.oauthTokenExpiresTime;
  }

  public String getOrgunitdn() {
    return orgunitdn;
  }

  public String getMatriculationnumber() {
    return matriculationnumber;
  }

  public Set<String> getRoles() {
    return roles;
  }

  public String getUserContext() {
    return userContext;
  }

  public Set<String> getUnigeChStudentCategory() {
    return unigeChStudentCategory;
  }

  public boolean hasRole(String role) {
    return roles.contains(role);
  }

  public boolean isEmployeeType(String employeeType) {
    return this.employeeType.contains(employeeType);
  }

  public boolean isUnigeChStudentCategory(String unigeChStudentCategory) {
    return this.unigeChStudentCategory.contains(unigeChStudentCategory);
  }

  public Map<String, String> getAllProperties() {
    return allProperties;
  }

  public boolean isEmpty() {
    return false;
  }

  public String getCnindividu() {
    return cnindividu;
  }

  @Override
  public String toString() {
    return "SsoUser{" + "userContext='" + userContext + '\'' + ", firstName='" + firstName + '\''
            + ", lastName='" + lastName + '\'' + ", email='" + email + '\'' + ", homeorganization='"
            + homeorganization + '\'' + ", homeorganizationtype='" + homeorganizationtype + '\''
            + ", affiliation=" + affiliation + ", ismemberOf=" + ismemberOf + ", employeeType="
            + employeeType + ", unigechoucode='" + unigechoucode + '\'' + ", unigechstudentoucode='"
            + unigechstudentoucode + '\'' + ", unigechemployeeoucode='" + unigechemployeeoucode
            + '\'' + ", uniqueId='" + uniqueId + '\'' + ", unigeChStudentCategory="
            + unigeChStudentCategory + ", orgunitdn='" + orgunitdn + '\''
            + ", matriculationnumber='" + matriculationnumber + '\'' + ", name='" + name + '\''
            + ", roles=" + roles + ", systemId=" + systemId + ", oauthToken=" + oauthToken
            + ", oauthTokenExpiresTime='" + oauthTokenExpiresTime + '\'' + ", cnindividu="
            + cnindividu + '}';
  }

}
