package ch.unige.solidify.rest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import com.fasterxml.jackson.annotation.JsonIgnore;

public class CollectionPage {
  private static final Logger logger = LoggerFactory.getLogger(CollectionPage.class);

  private long currentPage;
  private long sizePage;
  private long totalPages;
  private long totalItems;

  public static final int DEFAULT_SIZE_PAGE = 20;
  public static final int MAX_SIZE_PAGE = 2000;

  public <T> CollectionPage() {
    this.currentPage = 0;
    this.sizePage = DEFAULT_SIZE_PAGE;
    this.totalPages = 0;
    this.totalItems = 0;
  }

  public <T> CollectionPage(long nbItems) {
    this.currentPage = 0;
    this.sizePage = DEFAULT_SIZE_PAGE;
    this.totalPages = this.getPages(nbItems, DEFAULT_SIZE_PAGE);
    this.totalItems = nbItems;
  }

  public <T> CollectionPage(Page<T> page, Pageable pageable) {
    this.currentPage = pageable.getPageNumber();
    this.sizePage = pageable.getPageSize();
    this.totalPages = page.getTotalPages();
    this.totalItems = page.getTotalElements();
  }

  public <T> CollectionPage(long pageNum, long pageSize, long totalPages, long totalItems) {
    this.currentPage = pageNum;
    if (pageSize <= MAX_SIZE_PAGE) {
      this.sizePage = pageSize;
    } else {
      logger.warn(
              "The max value of page size must be under " + MAX_SIZE_PAGE + ", not " + pageSize);
      this.sizePage = MAX_SIZE_PAGE;
    }
    this.totalPages = totalPages;
    this.totalItems = totalItems;
  }

  public long getSizePage() {
    return sizePage;
  }

  public long getCurrentPage() {
    return currentPage;
  }

  public long getTotalPages() {
    return totalPages;
  }

  public long getTotalItems() {
    return totalItems;
  }

  private long getPages(long nbItems, long pageSize) {
    if (nbItems % pageSize == 0) {
      return nbItems / pageSize;
    } else {
      return nbItems / pageSize + 1;
    }
  }

  @JsonIgnore
  public long getCurrentItems() {
    if (totalItems < sizePage) {
      return totalItems;
    } else {
      if (currentPage + 1 == totalPages) {
        return totalItems - currentPage * sizePage;
      } else {
        return sizePage;
      }
    }
  }

  @JsonIgnore
  public long getPreviousItems() {
    return currentPage * sizePage;
  }

  @JsonIgnore
  public boolean hasPrevious() {
    return (totalItems != 0 && currentPage != 0);
  }

  @JsonIgnore
  public boolean hasNext() {
    return (totalItems != 0 && currentPage + 1 != totalPages);
  }
}
