package ch.unige.solidify.rest;

import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EmbeddedId;
import javax.persistence.EntityListeners;
import javax.persistence.MappedSuperclass;
import javax.validation.Valid;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonUnwrapped;

import ch.unige.solidify.listener.CacheListener;
import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.model.ChangeInfo;

@MappedSuperclass
@EntityListeners({ CacheListener.class })
public abstract class CompositeResource<K> extends ResourceBase {

  @EmbeddedId
  @JsonUnwrapped
  protected K compositeResId;

  public K getCompositeResId() {
    return compositeResId;
  }

  public void setCompositeResId(K compositeResId) {
    this.compositeResId = compositeResId;
  }

  @JsonIgnore
  public CacheMessage getCacheMessage() {
    return null;
  }

  // *************************************************************************************

  @AttributeOverrides({
          @AttributeOverride(name = "when", column = @Column(name = "creationWhen", length = 3)),
          @AttributeOverride(name = "who", column = @Column(name = "creationWho")) })
  @Embedded
  @Valid
  protected ChangeInfo creation = new ChangeInfo();

  @AttributeOverrides({
          @AttributeOverride(name = "when", column = @Column(name = "lastUpdateWhen", length = 3)),
          @AttributeOverride(name = "who", column = @Column(name = "lastUpdateWho")) })
  @Embedded
  @Valid
  protected ChangeInfo lastUpdate = new ChangeInfo();

  public void updateTime() {
    this.getLastUpdate().setWhen(OffsetDateTime.now(ZoneOffset.UTC));
  }

  @JsonIgnore
  public OffsetDateTime getCreationTime() {
    return this.getCreation().getWhen();
  }

  @JsonIgnore
  public OffsetDateTime getUpdateTime() {
    return this.getLastUpdate().getWhen();
  }

  @JsonIgnore
  public String getCreatedBy() {
    return this.getCreation().getWho();
  }

  public void setCreatedBy(String createdBy) {
    this.getCreation().setWho(createdBy);
    this.updateTime();
  }

  @JsonIgnore
  public String getUpdatedBy() {
    return this.getLastUpdate().getWho();
  }

  public void setUpdatedBy(String updatedBy) {
    this.getLastUpdate().setWho(updatedBy);
    this.updateTime();
  }

  public ChangeInfo getCreation() {
    return creation;
  }

  public ChangeInfo getLastUpdate() {
    return lastUpdate;
  }
}
