package ch.unige.solidify.rest;

import java.net.URI;
import java.util.List;
import java.util.Objects;

import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;

import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.message.ResourceCacheMessage;
import ch.unige.solidify.util.SearchCriteria;

public abstract class Resource<T> extends ResourceBase {

  public abstract String getResId();

  public abstract void setResId(String resId);

  @JsonIgnore
  public abstract Specification<T> getSpecification();

  public abstract void init();

  public abstract String managedBy();

  public void addLinks(WebMvcLinkBuilder linkBuilder, boolean mainRes, boolean subResOnly) {
    this.add(linkBuilder.slash(getResId()).withSelfRel());
    if (!subResOnly)
      this.add(
              (Tool.referenceLink(linkBuilder.slash(getResId()).toUriComponentsBuilder(), this.managedBy()))
                      .withSelfRel());
    this.add(linkBuilder.withRel(ActionName.LIST));
    if (mainRes)
      this.add((Tool.parentLink(linkBuilder.toUriComponentsBuilder())).withRel(ActionName.MODULE));
    else
      this.add((Tool.parentLink(linkBuilder.toUriComponentsBuilder())).withRel(ActionName.PARENT));
  }

  @JsonIgnore
  public URI getDownloadUri() {
    throw new UnsupportedOperationException();
  }

  @JsonIgnore
  public boolean isModifiable() {
    return true;
  }

  @JsonIgnore
  public boolean isDeletable() {
    return true;
  }

  protected <V extends Resource<V>> boolean exist(List<V> list, V item) {
    for (V i : list) {
      if (i.getResId().equals(item.getResId())) {
        return true;
      }
    }
    return false;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (!(obj instanceof Resource))
      return false;
    Resource<?> resource = (Resource<?>) obj;
    return super.equals(resource) && Objects.equals(getResId(), resource.getResId());
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), getResId());
  }

  @JsonIgnore
  public CacheMessage getCacheMessage() {
    return new ResourceCacheMessage(this.getClass(), this.getResId());
  }

  public Specification<T> getSearchSpecification(SearchCriteria criteria) {
    return null;
  }

  @JsonProperty
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private List<SearchCriteria> searchCriterias;

  @JsonIgnore
  public List<SearchCriteria> getSearchCriterias() {
    return searchCriterias;
  }

  @JsonProperty
  public void setSearchCriterias(List<SearchCriteria> searchCriterias) {
    this.searchCriterias = searchCriterias;
  }

  public String toString() {
    return this.getClass().getSimpleName() + ": resId=" + this.getResId();
  }

}