package ch.unige.solidify.rest;

import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EntityListeners;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.validation.Valid;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.listener.CacheListener;
import ch.unige.solidify.model.ChangeInfo;
import ch.unige.solidify.util.StringTool;

@MappedSuperclass
@EntityListeners({ CacheListener.class })
public abstract class ResourceNormalized<T> extends Resource<T> {
  @Id
  private String resId;

  protected ResourceNormalized() {
    super();
    resId = StringTool.generateResId();
  }

  public String getResId() {
    return resId;
  }

  public void setResId(String resId) {
    this.resId = resId;
  }

  // *************************************************************************************

  @AttributeOverrides({ @AttributeOverride(name = "when", column = @Column(name = "creationWhen", length = 3)),
          @AttributeOverride(name = "who", column = @Column(name = "creationWho")) })
  @Embedded
  @Valid
  protected ChangeInfo creation = new ChangeInfo();

  @AttributeOverrides({ @AttributeOverride(name = "when", column = @Column(name = "lastUpdateWhen", length = 3)),
          @AttributeOverride(name = "who", column = @Column(name = "lastUpdateWho")) })
  @Embedded
  @Valid
  protected ChangeInfo lastUpdate = new ChangeInfo();

  public void updateTime() {
    this.getLastUpdate().setWhen(OffsetDateTime.now(ZoneOffset.UTC));
  }

  @JsonIgnore
  public OffsetDateTime getCreationTime() {
    return this.getCreation().getWhen();
  }

  @JsonIgnore
  public OffsetDateTime getUpdateTime() {
    return this.getLastUpdate().getWhen();
  }

  @JsonIgnore
  public String getCreatedBy() {
    return this.getCreation().getWho();
  }

  public void setCreatedBy(String createdBy) {
    this.getCreation().setWho(createdBy);
    this.updateTime();
  }

  @JsonIgnore
  public String getUpdatedBy() {
    return this.getLastUpdate().getWho();
  }

  public void setUpdatedBy(String updatedBy) {
    this.getLastUpdate().setWho(updatedBy);
    this.updateTime();
  }

  public ChangeInfo getCreation() {
    return creation;
  }

  public ChangeInfo getLastUpdate() {
    return lastUpdate;
  }

}
