package ch.unige.solidify.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.CollectionPage;

public class CollectionTool {
  private static final Logger logger = LoggerFactory.getLogger(CollectionTool.class);

  private CollectionTool() {
    throw new IllegalStateException(SolidifyConstants.TOOL_CLASS);
  }

  public static <E> List<E> toList(Iterable<E> iterable) {
    if (iterable instanceof List) {
      return (List<E>) iterable;
    }
    ArrayList<E> list = new ArrayList<>();
    if (iterable != null) {
      for (E e : iterable) {
        list.add(e);
      }
    }
    return list;
  }

  private static ObjectMapper getObjectMapper(boolean failOnUnknown) {
    ObjectMapper mapper = new ObjectMapper();
    mapper.registerModule(new com.fasterxml.jackson.datatype.jsr310.JavaTimeModule());
    mapper.configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, false);
    mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, failOnUnknown);
    return mapper;
  }

  public static <T> List<T> getList(String jsonString, Class<T> itemClass) {
    List<T> result = new ArrayList<>();
    ObjectMapper mapper = getObjectMapper(false);

    try {
      JsonNode jsonPostResult = mapper.readTree(jsonString);
      String resourceArrayString = jsonPostResult.get("_data").toString();
      JavaType listItemType = mapper.getTypeFactory().constructParametricType(List.class,
              itemClass);
      result = mapper.readValue(resourceArrayString, listItemType);
    } catch (IOException e) {
      logger.error("Error in building list", e);
    }
    return result;
  }

  public static CollectionPage getPage(String jsonString) {
    CollectionPage result = null;
    ObjectMapper mapper = getObjectMapper(true);

    try {
      JsonNode jsonPostResult = mapper.readTree(jsonString);
      String pageString = jsonPostResult.get("_page").toString();
      result = mapper.readValue(pageString, CollectionPage.class);
    } catch (IOException e) {
      logger.error("Error in building list", e);
    }
    return result;
  }

  public static <T> List<T> initList(T firstValue) {
    return new ArrayList<>(Arrays.asList(firstValue));
  }
}
