package ch.unige.solidify.validation;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;

import ch.unige.solidify.util.ReflectionTool;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.util.StringTool;

/**
 * Validator used to check that if the field fieldName has the expected value fieldValue then the
 * field dependFieldName must be not null and not empty.
 */
public class AssertThatAnotherFieldHasValueValidator
        implements ConstraintValidator<AssertThatAnotherFieldHasValue, Object> {
  private String fieldName;
  private String expectedFieldValue;
  private String dependFieldName;

  @Override
  public void initialize(AssertThatAnotherFieldHasValue annotation) {
    fieldName = annotation.fieldName();
    expectedFieldValue = annotation.fieldValue();
    dependFieldName = annotation.dependFieldName();
  }

  @Override
  public boolean isValid(Object value, ConstraintValidatorContext ctx) {

    if (value == null) {
      return true;
    }

    try {
      String fieldValue = ReflectionTool.getProperty(value, fieldName);
      String dependFieldValue = ReflectionTool.getProperty(value, dependFieldName);

      if (expectedFieldValue.equals(fieldValue) && StringTool.isNullOrEmpty(dependFieldValue)) {

        ctx.disableDefaultConstraintViolation();
        ctx.buildConstraintViolationWithTemplate(ctx.getDefaultConstraintMessageTemplate())
                .addPropertyNode(dependFieldName).addConstraintViolation();

        return false;
      }

    } catch (NoSuchFieldException | IllegalAccessException ex) {
      throw new SolidifyRuntimeException("Error during validation", ex);
    }

    return true;
  }
}