package ch.unige.solidify.test.specification;

import ch.unige.solidify.test.config.MockUserJpaConfig;
import ch.unige.solidify.test.specification.dao.EntityMock;
import ch.unige.solidify.test.specification.dao.EntityMockRepository;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import javax.annotation.Resource;
import java.time.OffsetDateTime;
import java.util.Optional;

@RunWith(SpringRunner.class)
@DataJpaTest
@ContextConfiguration(classes = { MockUserJpaConfig.class })
public class ResourceTest {
  @Resource
  private EntityMockRepository entityMockRepository;

  @Before

  public void setup() {
    EntityMock entity = new EntityMock();
    entity.setEntityId("1");
    entity.setLabel("first");
    entity.setCreatedBy("unitTEST");
    entityMockRepository.save(entity);
  }

  @Test
  public void Test_insert_metaData() {

    Optional<EntityMock> entity2 = entityMockRepository.findById("1");
    Assert.assertNotNull(entity2.get().getCreationTime());
    Assert.assertNotNull(entity2.get().getUpdateTime());
  }

  @Test
  public void Test_update() {
    Optional<EntityMock> entity = entityMockRepository.findById("1");
    EntityMock entityMock = entity.get();
    OffsetDateTime oldUpdateAt = entityMock.getUpdateTime();
    entityMock.setLabel("changed");
    entityMockRepository.save(entityMock);
    Optional<EntityMock> entity2 = entityMockRepository.findById("1");
    Assert.assertEquals("changed", entity2.get().getLabel());
  }

  @Test
  public void Test_delete() {
    entityMockRepository.deleteById("1");
    Optional<EntityMock> entity2 = entityMockRepository.findById("1");
    Assert.assertFalse(entity2.isPresent());
  }
}
