package ch.unige.solidify.test.util;

import ch.unige.solidify.util.StringCipherTools;
import org.junit.Test;

import java.nio.charset.Charset;
import java.util.Random;
import java.util.concurrent.ThreadLocalRandom;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

public class StringCipherToolsTest {

  private static final int NB_TRIES = 1_000;
  private static final int MIN_PLAIN_TEXT_LENGTH = 1;
  private static final int MAX_PLAIN_TEXT_LENGTH = 200;

  @Test
  public void cipherTestWithGeneratedSalt() {
    for (int i = 0; i < NB_TRIES; i++) {
      String plainText = generateRandomPlainText();
      String password = generateRandomPassword();
      String cipherText = StringCipherTools.encrypt(plainText, password, 8);
      String decryptedText = StringCipherTools.decrypt(cipherText, password, 8);
      assertEquals("Plain text (" + plainText + ") differs from decrypted text (" + decryptedText + ")", plainText, decryptedText);
      assertNotEquals("Plain text (" + plainText + ") equals cipher text (" + cipherText + ")", plainText, cipherText);
    }
  }

  @Test
  public void cipherTestWithPredefinedSalt() {
    for (int i = 0; i < NB_TRIES; i++) {
      String plainText = generateRandomPlainText();
      String password = generateRandomPassword();
      String salt = generateRandomSalt();
      String cipherText = StringCipherTools.encrypt(plainText, password, salt.getBytes());
      String decryptedText = StringCipherTools.decrypt(cipherText, password, salt.getBytes().length);
      assertEquals("Plain text (" + plainText + ") differs from decrypted text (" + decryptedText + ")", plainText, decryptedText);
      assertNotEquals("Plain text (" + plainText + ") equals cipher text (" + cipherText + ")", plainText, cipherText);
    }
  }

  @Test
  public void undeterministCipher() {
    for (int i = 0; i < NB_TRIES; i++) {
      String plainText = generateRandomPlainText();
      String salt = generateRandomSalt();
      String password = generateRandomPassword();
      String cipherText1 = StringCipherTools.encrypt(plainText, password, salt.getBytes());
      String cipherText2 = StringCipherTools.encrypt(plainText, password, salt.getBytes());
      assertNotEquals("Two encryption of the plain text (" + plainText + ") gives the same cipher", cipherText1, cipherText2);
    }
  }

  private String generateRandomPlainText() {
    int plainTextLength = ThreadLocalRandom.current().nextInt(MIN_PLAIN_TEXT_LENGTH, MAX_PLAIN_TEXT_LENGTH + 1);
    byte[] plainTextBytes = new byte[plainTextLength];
    new Random().nextBytes(plainTextBytes);
    return new String(plainTextBytes, Charset.forName("UTF-8"));
  }

  private String generateRandomSalt() {
    byte[] saltBytes = new byte[8];
    new Random().nextBytes(saltBytes);
    return new String(saltBytes, Charset.forName("UTF-8"));
  }

  private String generateRandomPassword() {
    return generateRandomPlainText();
  }
}
