package ch.unige.solidify.util;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyRuntimeException;

public class JSONTool {
  private static final Logger log = LoggerFactory.getLogger(JSONTool.class);

  /**
   * This constructor should not be called
   */
  private JSONTool() {
    throw new IllegalStateException(SolidifyConstants.TOOL_CLASS);
  }

  @SuppressWarnings("unchecked")
  public static Map<String, Object> convertJson2Map(String s) {
    try {
      return new ObjectMapper().readValue(s, HashMap.class);
    } catch (IOException e) {
      log.error("Error while converting Json to Map {}", s);
    }
    return new HashMap<>();
  }

  public static String convert2JsonString(Object object) {
    try {
      ObjectMapper mapper = new ObjectMapper();
      mapper.enable(SerializationFeature.INDENT_OUTPUT);
      mapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);
      StringWriter sw = new StringWriter();
      mapper.writeValue(sw, object);
      return sw.toString();
    } catch (IOException e) {
      throw new SolidifyRuntimeException(e.getMessage());
    }
  }

  public static JSONObject convert2Json(Object object) {
    try {
      return new JSONObject(convert2JsonString(object));
    } catch (JSONException e) {
      throw new SolidifyRuntimeException(e.getMessage());
    }
  }

  public static JSONArray convert2JsonArray(Object object) {
    try {
      return new JSONArray(convert2JsonString(object));
    } catch (JSONException e) {
      throw new SolidifyRuntimeException(e.getMessage());
    }
  }

  public static String convert2SortedJsonString(JSONObject json) {
    try {
      ObjectMapper mapper = new ObjectMapper().copy()
              .configure(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS, true)
              .configure(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY, true)
              .configure(SerializationFeature.INDENT_OUTPUT, true);
      Object sortedJson = mapper.readValue(json.toString(2), Object.class);
      return mapper.writeValueAsString(sortedJson);
    } catch (JSONException | IOException e) {
      throw new SolidifyRuntimeException(e.getMessage());
    }
  }

}
