package ch.unige.solidify.util;

import java.lang.reflect.Field;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.unige.solidify.SolidifyConstants;

public class ReflectionTool {
  private static final Logger log = LoggerFactory.getLogger(ReflectionTool.class);

  /**
   * This constructor should not be called
   */
  private ReflectionTool() {
    throw new IllegalStateException(SolidifyConstants.TOOL_CLASS);
  }

  public static String getCurrentMethodName() {
    return ReflectionTool.getMethodName(3);
  }

  public static String getMethodName(int stackTraceIndex) {
    return Thread.currentThread().getStackTrace()[stackTraceIndex].getMethodName();
  }

  public static String getCurrentSetterVariableName() {
    String currentMethodName = ReflectionTool.getMethodName(3);

    if (currentMethodName.startsWith("set") && Character.isUpperCase(currentMethodName.charAt(3))) {
      return currentMethodName.substring(3, 4).toLowerCase() + currentMethodName.substring(4);
    } else {
      return null;
    }
  }

  /**
   * Returns the first field in the hierarchy for the specified name
   */
  public static Field getField(Object object, String name) throws NoSuchFieldException {
    Class<?> clazz = object.getClass();
    Field field = null;

    while (clazz != null && field == null) {

      try {
        field = clazz.getDeclaredField(name);
      } catch (NoSuchFieldException e) {
        log.debug("Field not found", e);
      }

      clazz = clazz.getSuperclass();
    }

    if (field == null) {
      throw new NoSuchFieldException(name);
    }

    return field;
  }

  /**
   * Return the value of the specified property of the specified object as a String
   *
   * @param object
   *          Object whose property is to be extracted
   * @param name
   *          Name of the property to be extracted
   * @return String The property's value, converted to a String
   * @throws NoSuchFieldException
   * @throws IllegalAccessException
   */
  public static String getProperty(Object object, String name)
          throws NoSuchFieldException, IllegalAccessException {
    Field field = ReflectionTool.getField(object, name);
    field.setAccessible(true);
    Object valueObj = field.get(object);
    return (valueObj != null) ? valueObj.toString() : null;
  }
}
