package ch.unige.solidify.test.util;

import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.junit4.SpringRunner;

import ch.unige.solidify.util.JSONTool;

@RunWith(SpringRunner.class)
public class JSONTest {

  @Test
  public void convert2JsonStringTest() {

    // Line separator
    String lineSep = System.lineSeparator();

    // long
    Long lvalue = Long.valueOf(10);
    String newValue = JSONTool.convert2JsonString(lvalue.longValue());
    assertTrue("Long conversion: ", newValue.equals(lvalue.toString()));

    // String
    String svalue = "string";
    newValue = JSONTool.convert2JsonString(svalue);
    assertTrue("String conversion: ", newValue.equals("\"" + svalue + "\""));

    // String[]
    String[] sarray = { "string1", "string2", "string3" };
    String arrayString = "[ \"string1\", \"string2\", \"string3\" ]";
    newValue = JSONTool.convert2JsonString(sarray);
    assertTrue("String array conversion: ", newValue.equals(arrayString));

    // List<String>
    List<String> larray = new ArrayList<>();
    larray.add("string1");
    larray.add("string2");
    larray.add("string3");
    newValue = JSONTool.convert2JsonString(larray);
    assertTrue("String list conversion: ", newValue.equals(arrayString));

    // PersonTest object
    PersonTest person = new PersonTest();
    String personString = "{" + lineSep + "  \"firstName\" : \"My First Name\"," + lineSep
            + "  \"lastName\" : \"My Last Name\"," + lineSep + "  \"zipCode\" : 74160" + lineSep
            + "}";
    newValue = JSONTool.convert2JsonString(person);
    assertTrue("Person conversion: ", newValue.equals(personString));

    // List<PersonTest>
    List<PersonTest> parray = new ArrayList<>();
    parray.add(new PersonTest("Astérix"));
    parray.add(new PersonTest("Obélix"));
    parray.add(new PersonTest("Idéfix"));
    String personList = "[ {" + lineSep + "  \"firstName\" : \"My First Name Astérix\"," + lineSep
            + "  \"lastName\" : \"My Last Name Astérix\"," + lineSep + "  \"zipCode\" : 74160"
            + lineSep + "}, {" + lineSep + "  \"firstName\" : \"My First Name Obélix\"," + lineSep
            + "  \"lastName\" : \"My Last Name Obélix\"," + lineSep + "  \"zipCode\" : 74160"
            + lineSep + "}, {" + lineSep + "  \"firstName\" : \"My First Name Idéfix\"," + lineSep
            + "  \"lastName\" : \"My Last Name Idéfix\"," + lineSep + "  \"zipCode\" : 74160"
            + lineSep + "} ]";
    newValue = JSONTool.convert2JsonString(parray);
    assertTrue("Person list conversion: ", newValue.equals(personList));

  }

  @Test
  public void convert2JsonObjectTest() {

    // PersonTest object
    PersonTest person = new PersonTest();
    String personString = "{\"firstName\":\"My First Name\",\"lastName\":\"My Last Name\",\"zipCode\":74160}";
    JSONObject newValue = JSONTool.convert2Json(person);
    assertTrue("Person conversion: ", newValue.toString().equals(personString));
  }

  @Test
  public void convert2JsonArrayTest() {

    // List<PersonTest>
    List<PersonTest> parray = new ArrayList<>();
    parray.add(new PersonTest("Astérix"));
    parray.add(new PersonTest("Obélix"));
    parray.add(new PersonTest("Idéfix"));
    String personList = "["
            + "{\"firstName\":\"My First Name Astérix\",\"lastName\":\"My Last Name Astérix\",\"zipCode\":74160},"
            + "{\"firstName\":\"My First Name Obélix\",\"lastName\":\"My Last Name Obélix\",\"zipCode\":74160},"
            + "{\"firstName\":\"My First Name Idéfix\",\"lastName\":\"My Last Name Idéfix\",\"zipCode\":74160}"
            + "]";
    JSONArray newValue = JSONTool.convert2JsonArray(parray);
    assertTrue("Person list conversion: ", newValue.toString().equals(personList));

  }
}
